#!/usr/local/bin/python

# PAMELA, by Myles Strous, myles@fcmail.com, Copyright 2001, Myles Strous
# 
# Python-based Accessory for Moho Enhanced Lipsynch Activities
# 
# This is a lip-synch aid written for use with Lost Marble Moho
# a great 2D character animation program (see http://www.lostmarble.com/)
#
# This script is written in Python (see http://www.python.org/)
# 
#  Credits page in HTML docs : 
#      Python license
#      Guido and others for writing Python and its modules, their companies for paying them to do so (where appropriate)
#      Tcl/Tk license
#      Snack license when included - GPL
#      Snack author for Snack
#      CMU Pronouncing Dictionary
#      soundex module
#      general Tkinter - /F (PDF intro to Tkinter, newsgroup postings) , Grayson (book and newsgroup postings), Guido, Matt (in demo source code), comp.lang.python in general
#      the author of LipSync, Hash Inc, the author of Magpie, for showing me what was needed
#      the person/s who wrote the RIFF and WAV docs
#      Gordon McMillan(sp) for his installer (assuming I can get it working)
#      whichever freeware Windows installation program I end up using
#      Last but not least, Mike Clifton and LostMarble for Moho (and Switch layers in particular, and making Switch data an ASCII format and documenting it)
#
# ==============================================================
# History :
#
#   December 2001 - version 0.1 released
#
#   March 2002 - version 0.2 released
#       New features : 
#           playback preview - phoneme-by-phoneme sound with image preview
#           phoneme sound when phoneme selected on canvas
#           import Switch data files
#           optional phonetic breakdown with reduced phonemes
#               keep front phoneme, last phoneme, 
#               keep if FV, MBP, WQ, 
#               keep if primary (or secondary ?) emphasised vowel
#               otherwise delete
#       Fixed :
#           Insert phoneme uses single phoneme dialog box
#           Replace phoneme uses single phoneme dialog box
#       more sample MOT files : Preston2Preston, Magpie2Preston
#       more documentation about MOT files
#
#   March 2002 - version 0.21 released (just the exe and the main py file) 
#       The canvas now starts at a larger size horizontally
#       The canvas now resizes horizontally with the window
#       Fixed : changing/editing a phoneme didn't export
#       Fixed : loading the tutorial from program (help menu) should now work (on Windows, anyway)
#       Changed : some of the internal event handling, no visible user effect, 
#           but hopefully better memory usage
#
#   April 2002 - version 0.3
#       Added pause indicator ("_") to insert spacing to improve automatic matching
#       Removed automatic breakdown when preferences changed
#       Started moving text strings to single file (strings.py) - enabling language customisation
#       phoneme example usage help in phoneme selection box
#       Phoneme list box under Help : Phoneme List
#       Toolbar
#             phoneme insertion
#             MOT file selection
#             preview
#       Moved some code from PAMELA.py to pamgui.py (no visible change to users)
#
# ==============================================================
#
#   create a proper Windows installation distribution,
#       automatically create Start Menu entries,
#       optional desktop shortcut
#
# ==============================================================
#
#   TO DO for 0.4
#
#   dictionary editing - add, change, ??delete??
#   internal improvements to code - effects not visible to users 
#       - unit testing where possible
#       - comments and internal code documentation
#       - more error messages to user where circumstances are not as expected
#   continue transfer of text strings in all code files to a single file (pamela.strings), to allow
#       for easier language customisation (ask users for translations)
#   simplified phoneme set without vowel emphasis ??


# standard modules
import bsddb, dbhash
import string, os.path
# from Tkinter import *
import Canvas, tkMessageBox, tkFileDialog, tkSimpleDialog
import Tkinter

# PAMELA modules
import pamgui, prefgui, pamfile
import motedit, lookupword, export
import mysound
import spanish_breakdown
import breakdowngui
# import getmot
import strings
strings = strings.messages

# import translategui

# ==============================================================

class PAMELA(pamgui.PAMELAgui):
    """inherits from pamgui, a frame with menus and statusbar"""

    def __init__(self, parent=None):
        pamgui.PAMELAgui.__init__(self, parent)  # calls create widgets
        self.pack(expand=Tkinter.YES, fill=Tkinter.BOTH)

        self.parent = parent
        self.phonetictext = []  # the phonetically broken down text, as a simple list of phonemes
        self.citems = {}  # key is the line tag number, value is the text/phoneme tag number
        self.timeitems = {}  # key is the text/phoneme tag number, value is the an [xpos, phoneme] pair

        self.pixelsperframe = 15  # spacing between frame markers

        self.readpreferences()

        self.remember = None
        self.endbuffer = 300  # allow a bit of extra space at the end of the canvas


    def readpreferences(self, filename=None) :
        """Reads the preferences file when the program starts.

        UNIT TESTING : 
        run function, print variables
        reads filename
        no filename, reads PAMELA.prf

        add error messages to user : 
            cannot find PAMELA.prf
            cannot find sample.mot
        """
        if not filename :
            # filename = './PAMELA.prf'
            filename = 'PAMELA.prf'
        if not os.path.exists(filename):
            tkMessageBox.showinfo(strings["No preferences file"], strings["cannot find preferences file : %s"] % filename)
        prefsdata = pamfile.pamfile(filename)
        self.fps = int(prefsdata['Preferences']['framespersecond'])
        self.framesperphoneme = int(prefsdata['Preferences']['framesperphoneme'])
        self.pixelspersecond = self.pixelsperframe * self.fps
        self.translation_file = prefsdata['Preferences']['translationfile']
        if not os.path.exists(self.translation_file):
            tkMessageBox.showinfo(strings["No translation file"], strings["cannot find translation (mot) file : %s\n"] % self.translation_file)
        self.getPreviewImages() # get preview images corresponding to this translation file
        self.currentMOTfile.set(self.translation_file)  # sets the option menu in the toolbar


    def createWidgets(self):
        """Creates the canvas and preview windows.

        Menus, status bar and text entry are created in pamgui
        """
        pamgui.PAMELAgui.createWidgets(self)  # menu bar, status bar, text entry
        self.sb = Tkinter.Scrollbar(self, orient="horizontal", jump=1)
        self.blank = Tkinter.PhotoImage(file="preview"+os.sep+"blank.gif")
        self.video = Tkinter.Label(self, image=self.blank)
        # self.video.pack(side=Tkinter.RIGHT, expand=Tkinter.YES, fill=Tkinter.BOTH, padx=5, pady=5)
        self.video.pack(side=Tkinter.RIGHT, expand=Tkinter.NO, padx=5, pady=5)
        self.contextmessage[self.video] = strings["This is the image area for preview playback and phoneme preview"]
        self.video.bind("<Enter>", self.contextinfo)
        canvas_height = 160
        self.display = Tkinter.Canvas(self, height=canvas_height, bg="white", xscrollcommand=self.sb.set, width=600)
        self.sb.config(command=self.display.xview)
        self.sb.pack(side=Tkinter.BOTTOM, fill=Tkinter.X, padx=5)
        self.display.pack(expand=Tkinter.YES, fill=Tkinter.X, padx=5)
        self.contextmessage[self.display] = strings["This is the canvas - it displays the waveform, frame markers, and movable phonemes"]
        self.display.bind("<Enter>", self.contextinfo)
        self.contextmessage[self.sb] = strings["This is the scrollbar for scrolling along the waveform and phonemes"]
        self.sb.bind("<Enter>", self.contextinfo)
        self.display.bind("<Button-1>", self.playphoneme)
        self.display.bind("<Button-3>", self.rmb)
        self.display.bind("<B1-Motion>", self.slidehorizontal)

    def rmb(self, event):
        handle = self.display.find_withtag(Tkinter.CURRENT)[0]
        if handle in self.timeitems.keys():  # if it's a phoneme text item
            self.edittext(event)
        elif handle in self.citems.keys():  # if it's a line item
            self.deletephonemefromline(event)

    def getPreviewImages(self):
        """reads all the gif images in the preview folder, stores them in a dictionary in PhotoImage format,
        uses the current mot file to create a phoneme to image translation dictionary

        UNIT TESTING (workaround the Tkinter requirement ?):
        run function, then print imagelist and translation dictionary
        
        error messages to user : cannot find preview folder, cannot find blank.gif
        """
        self.imagelist = {} # filename minus extension : photoimage handle
        import glob, ConfigParser
        filelist = glob.glob('preview'+os.sep+'*.gif')
        for imagefile in filelist:
           img = Tkinter.PhotoImage(file=imagefile)
           imagename = os.path.splitext(os.path.split(imagefile)[1])[0]
           self.imagelist[imagename] = img
        self.translation = {}
        t = ConfigParser.ConfigParser()
        t.read(self.translation_file)
        for phoneme in t.options('Translations') :
                self.translation[string.upper(phoneme)] = t.get('Translations', phoneme)


# ==============================================================

    def debugcitems(self) :
        print self.citems

    def debugtimeitems(self) :
        print self.timeitems

    def debugphonetictext(self) :
        print self.phonetictext

    def debugspeech(self):
        speech = self.timeitems.items()
        speech.sort()
        print speech

    def debugframes(self):
        speech = self.timeitems.values()
        speech.sort()
        for eachframe in speech:
            print int(eachframe[0] * self.fps / self.pixelspersecond)+1, eachframe[1]

    def debugoutput(self) :
        x = self.timeitems.values()
        x.sort()
        print x

# ==============================================================

    def preferences(self) :
        """runs the preference dialog to change preferences, saves the new preferences, re-reads them, 
        re-runs the phonetic breakdown if the FPS or framesperphoneme values changes and a project is loaded
        """
        past_fps = self.fps
        past_framesperphoneme = self.framesperphoneme
        prefgui.saveprefs(self.fps, self.framesperphoneme, self.translation_file) # save current to ./PAMELA.prf
        p = prefgui.preferencesDialog(self) # get new, which saves changed values
        pr = p.result
        if pr != None : # changed preferences
            self.fps = pr['fps']
            self.framesperphoneme = pr['framesperphoneme']
            self.translation_file = pr['translation_file']
            self.pixelspersecond = self.pixelsperframe * self.fps
            prefgui.saveprefs(self.fps, self.framesperphoneme, self.translation_file)
            self.readpreferences()
            # if hasattr(self, 'soundfile') and (past_fps != self.fps or past_framesperphoneme != self.framesperphoneme):
            #    self.phoneticBreakdown()

    def newmot(self, nuMOTfile):  # used by version 0.3 toolbar
        self.translation_file = nuMOTfile.get()
        prefgui.saveprefs(self.fps, self.framesperphoneme, self.translation_file)
        self.readpreferences()

#    def choosemot(self):
#        numot = getmot.getmot(self)
#        if numot:
#            self.translation_file = numot
#            # change the preferences and preferences file
#            prefgui.saveprefs(self.fps, self.framesperphoneme, self.translation_file)
#            self.readpreferences()


    def editphonemetranslation(self) :
        """loads up the small internal editor to change or create the mot phoneme translation file"""
        if self.translation_file and os.path.exists(self.translation_file):
            t = motedit.MotEdit(self, self.translation_file)
        else:
            t = motedit.MotEdit(self)
# ==============================================================

    def addphonemetocanvas(self, phoneme, xpos):
        """adds a single phoneme to the canvas at a given position

        XXX change these tag_bind-ings to the canvas instead, 
        then get nearest object that is a line/phoneme(not a frame marker or the waveform),
        then call the appropriate function with that object
        """
        linei = self.display.create_line(xpos, 0, xpos, self.display.winfo_height())
        self.display.tag_bind(linei, "<Enter>", self.gored)
        # self.display.tag_bind(linei, "<Button-3>", self.deletephonemefromline) # test line
        self.display.tag_bind(linei, "<Leave>", self.go_redtoblack)
        # self.display.tag_bind(linei, "<B1-Motion>", self.slidehorizontal)
        texti = self.display.create_text(xpos+3, self.display.winfo_height()-10, text=phoneme, anchor=Tkinter.W)
        # self.display.tag_bind(texti, "<Button-1>", self.playphoneme)
        # self.display.tag_bind(texti, "<Button-3>", self.edittext)
        self.display.tag_bind(texti, "<Enter>", self.ontext)
        self.display.tag_bind(texti, "<Leave>", self.go_bluetoblack)
        self.citems[linei] = texti
        self.timeitems[texti] = [xpos, phoneme]
        pass

# ==============================================================


    def insertphoneme(self):
        """asks for a new phoneme to insert,
        works out the current canvas details, 
        calls addphonemetocanvas to place the phoneme in the center.
        """
        simple = 0
        if simple:
            insertph = tkSimpleDialog.askstring(strings['Phoneme to insert ?'], strings['Phoneme to insert ?'])
            first, last = self.sb.get()
            midway = self.display.canvasx(int(self.display.winfo_width()/2))
            midx = midway - (midway % self.separation)
            self.addphonemetocanvas(insertph, midx)
        else:
            t = breakdowngui.singlephonemeDialog(self)
            wd = t.result
            if wd :
                midway = self.display.canvasx(int(self.display.winfo_width()/2))
                midx = midway - (midway % self.separation)
                self.addphonemetocanvas(wd, midx)


    def insertphoneme__from_toolbar(self):
        nuphoneme = self.phonemeToInsert.get()
        stresstrans = {strings["Vowel - no stress"]:"0", strings["Vowel - primary stress"]:"1", strings["Vowel - secondary stress"]:"2"}
        if nuphoneme[0] in "AEIOU":
            stress = self.phonemeStressToInsert.get()
            nuphoneme = nuphoneme + stresstrans[stress]
        midway = self.display.canvasx(int(self.display.winfo_width()/2))
        midx = midway - (midway % self.separation)
        self.addphonemetocanvas(nuphoneme, midx)


    def deletephonemefromline(self, event):
        """deletes the line and the associated phoneme from the canvas and the lists
        """
        handle = self.display.find_withtag(Tkinter.CURRENT)
        associatedtext = self.citems[handle[0]]
        self.display.delete(associatedtext)
        self.display.delete(Tkinter.CURRENT)
        del self.citems[handle[0]]
        del self.timeitems[associatedtext]

    def gored(self, event) :
        """changes the current line and the associated text to a red colour
        """
        self.display.itemconfig(Tkinter.CURRENT, fill="red")
        handle = self.display.find_withtag(Tkinter.CURRENT)
        associatedtext = self.citems[handle[0]]
        self.display.itemconfig(associatedtext, fill="red")
        self.writeStatusBar(strings["Click and drag the line to move, right-click the line to delete this phoneme"])

    def go_redtoblack(self, event) :
        """changes the current line and the associated text back to a black colour
        """
        self.display.itemconfig(Tkinter.CURRENT, fill="black")
        handle = self.display.find_withtag(Tkinter.CURRENT)
        associatedtext = self.citems[handle[0]]
        self.display.itemconfig(associatedtext, fill="black")
        self.writeStatusBar("")


    def slidehorizontal(self, event) :
        """slides the current line and associated text according to LMB-drag, 
        uses self.separation to restrict dragging to discrete steps
        """
        
        handle = self.display.find_withtag(Tkinter.CURRENT)[0]
        if handle in self.citems.keys(): # if it's a line
            if self.display.canvasx(event.x) < 0 :
                newx = 0
            else :
                newx = self.display.canvasx(event.x, self.separation)
            self.display.coords(Tkinter.CURRENT, newx, self.display.winfo_height(), newx, 0)
            handle = self.display.find_withtag(Tkinter.CURRENT)
            associatedtext = self.citems[handle[0]]
            self.display.coords(associatedtext, newx+3, self.display.winfo_height()-10)
            self.timeitems[associatedtext][0] =newx

    def ontext(self, event) :
        """this function is called when the mouse is enters a phoneme, 
        and updates the statusbar and colours the text blue
        """
        self.writeStatusBar(strings["Click the text to play the phoneme area, right-click to change the phoneme"])
        self.display.itemconfig(Tkinter.CURRENT, fill="blue")
        self.remember =  self.display.find_withtag(Tkinter.CURRENT)

    def go_bluetoblack(self, event) :
        """this function is called when the mouse leaves a phoneme, 
        it changes the phoneme colour back to black and blanks the statusbar
        """
        self.display.itemconfig(self.remember, fill="black")
        self.writeStatusBar("")

    def playphoneme(self, event) :
        """this function is called when the mouse (LMB) is clicked on a phoneme, 
        it plays the sound section selected by the appropriate phoneme markers, 
        and updates the preview image
        """
        handle = self.display.find_withtag(Tkinter.CURRENT)
        if handle:
            associatedtext = handle[0]
            if associatedtext in self.timeitems.keys(): # if it's a phoneme text item
                phoneme = self.timeitems[associatedtext][1]
                all = self.timeitems.values() # list of [xpos, phoneme] pairs
                all.sort()
                if string.lower(phoneme) != all[-1][1]: # we're not at the last phoneme - the "end" marker
                    startpos = self.timeitems[associatedtext][0] # actually gets a pixel position
                    all2 = map(lambda n:n[0], all) # creates a list of x positions
                    x = all2.index(startpos) # gets the index where this phoneme starts
                    endpos = all2[x+1] # gets the pixel position of the next phoneme by index
                    # Snack time units are in terms of samples, not seconds
                    endframe = int(round(self.snd.framerate * endpos / float(self.pixelspersecond)))
                    startframe = int(round(self.snd.framerate * startpos/float(self.pixelspersecond)))
                    self.snd.playsound(start=startframe, end=endframe)
                    try:
                        self.video.configure(image = self.imagelist[self.translation[self.timeitems[associatedtext][1]]])
                        # self.video.update()
                    except KeyError:  # couldn't find that image
                        pass


    def edittext(self, event) :
        """this function is called when the mouse is right-clicked on a phoneme, and changes the text
        on the current phoneme
        """
        simple = 0
        if simple:
            wd = tkSimpleDialog.askstring(strings["Change phoneme to what ?"], strings["change phoneme to what ?"])
            if wd != None :
                self.display.itemconfig(Tkinter.CURRENT, text=wd)
        else:
            t = breakdowngui.singlephonemeDialog(self)
            wd = t.result
            if wd :
                self.display.itemconfig(Tkinter.CURRENT, text=wd)
        text_tag_number =  self.display.find_withtag(Tkinter.CURRENT)[0]
        self.timeitems[text_tag_number][1] = wd
        
# ==============================================================

    def playWAV(self) :
        """plays the entire WAV file currently loaded

        XXX - what happens when there is no sound file loaded yet ?
        Probably crashes - FIXME
        """
        self.snd.playsound()

    def playPreview(self) :
        """shows the image preview and plays the sound section associated with each phoneme
        """
        testpreview = 1
        if testpreview == 0:
            pass
        else :
            self.setup_playlist()
            for each in range(len(self.playlist)):
                self.shownext(each)  # show the preview image
                self.playnext(each)  # play the sound section
            self.video.configure(image = self.blank)  # blanks the preview window at the end

    def shownext(self, each):
        """shows the preview image associated with the next phoneme in the preview list
        """
        try:
            self.video.configure(image = self.imagelist[self.playlist[each][2]])
            self.video.update()
        except KeyError:  # couldn't find that image
            pass

    def playnext(self, each):
        """plays the next sound section in the preview list

        blocks the program from doing anything else while the sound is playing
        """
        self.snd.playsound(start=self.playlist[each][0], end=self.playlist[each][1], blocking=1)


    def setup_playlist(self) :
        """ use timeitems to set up a preview playlist, for each phoneme in timeitems, of 
        (startframe, endframe, translated_phoneme_image) tuples 
        where startframe and endframe are sound file frames
        """
        self.playlist = []
        timevalues = self.timeitems.values()
        timevalues.sort()
        for each in range(len(timevalues)):
            if timevalues[each][1] == "end":
                break
            startframe = int(round(self.snd.framerate * timevalues[each][0]/float(self.pixelspersecond)))
            endframe = int(round(self.snd.framerate * timevalues[each+1][0]/float(self.pixelspersecond)))
            starttime = timevalues[each][0] / float(self.pixelspersecond)
            endtime = timevalues[each+1][0] / float(self.pixelspersecond)
            try:
                phoneme_to_show = self.translation[timevalues[each][1]]
            except KeyError:  # don't have a translation for that phoneme
                # tkMessageBox.showinfo("Error", "Translation file doesn't have a corresponding value")
                phoneme_to_show = ""
            # self.playlist.append((startframe,endframe,phoneme_to_show, starttime, endtime))
            self.playlist.append((startframe,endframe,phoneme_to_show))


    def _opensoundfile(self) :
        """opens a sound file, displays the waveform on the canvas, sets up frame markers, where 
        frames refers to frames in Moho, and creates an "end" pseudo-phoneme on the canvas
        """
        self.snd = mysound.mySound(self.soundfile)
        self.snd.drawsound(self.display, self.pixelspersecond)
        self.display.config(scrollregion=(0,0,self.snd.width+self.endbuffer,self.display.winfo_height()))
        numberofmarkers = ((self.snd.width+self.endbuffer)*self.fps)/self.pixelspersecond
        self.separation = self.pixelspersecond / self.fps

        for i in range(1,numberofmarkers) :
            x = self.display.create_line(i*self.separation, self.display.winfo_height()-40, i*self.separation, self.display.winfo_height()-20)
        self.addphonemetocanvas('end', (i-3)*self.separation)

    def addwordstocanvas(self) :
        """takes the broken down list of phonemes, and places them evenly on the canvas,
        resetting the scrollregion of the phonemes go past the end of the waveform and canvas
        """
        i = 1
        self.separation = self.pixelspersecond / self.fps  # space for each frame
        spacer = self.separation*self.framesperphoneme  # space for each phoneme
        for phoneme in self.phonetictext  :
            if phoneme != "_":
                self.addphonemetocanvas(phoneme, i*spacer)
            i = i+1
        # self.addphonemetocanvas('end', i*spacer)
        self.display.config(scrollregion=(0,0,max(self.snd.width+self.endbuffer, (i*spacer)+self.endbuffer),self.display.winfo_height()))

    def importSwitchFile(self):
        """clears all current phonemes, loads a Moho Switch file, 
        and places the phonemes from the Switch file onto the canvas

        If there is no soundfile already loaded, gives an error message instead
        """
        if hasattr(self, 'soundfile'):
            self.phonetictext = []
            self.display.delete(Tkinter.ALL) # clear the entire canvas
            self.phonetictext = []
            self.citems = {}
            self.timeitems = {}
            self._opensoundfile() # recreates the waveform and tickmarks

            fn = tkFileDialog.askopenfilename(filetypes=[('Moho Switch data files', '*.dat')])
            if fn :
                switchfile = open(fn)
                for line in switchfile.xreadlines():
                    if not line:
                        break  # EOF
                    elif line.strip() == "MohoSwitch1":
                        pass
                    elif line.strip() == '' :  # empty line
                        pass 
                    elif len(line.strip().split()) == 1: # frame number, no phoneme layer
                        pass
                    else:
                        frame, phoneme = map(string.strip, line.split())
                        # insert onto canvas and into items
                        pixels = (int(frame)-1) * self.pixelspersecond / self.fps
                        self.addphonemetocanvas(phoneme, pixels)
                switchfile.close()
        else:
            tkMessageBox.showinfo(strings["No sound file"], strings["No sound file loaded - start a new Project first"])

    def simplifiedPhoneticBreakdown(self):
        """calls lookupword.simplifiedbreakdowntext to breakdown the sentence in self.textentry
        into a list of phonemes, then loads them onto the canvas

        If there is no soundfile already loaded, gives an error message instead
        """
        if hasattr(self, 'soundfile'):
            self.phonetictext = []
            self.display.delete(Tkinter.ALL) # clear the entire canvas
            self.phonetictext = []
            self.citems = {}
            self.timeitems = {}
            self._opensoundfile() # recreates the waveform and tickmarks

            # get the current text to break down
            normaltext = self.textentry.get()
            self.textentry.update_idletasks()

            # break it down
            self.phonetictext = lookupword.simplifiedbreakdowntext(normaltext, self)

            # add it to the canvas
            self.addwordstocanvas()
        else:
            tkMessageBox.showinfo(strings["No sound file"], strings["No sound file loaded - start a new Project first"])

    def spanishPhoneticBreakdown(self):
        """calls spanish_breakdown.breakdowntext to breakdown the sentence in self.textentry
        into a list of phonemes, then loads them onto the canvas

        If there is no soundfile already loaded, gives an error message instead
        """
        if hasattr(self, 'soundfile'):
            self.phonetictext = []
            self.display.delete(Tkinter.ALL) # clear the entire canvas
            self.phonetictext = []
            self.citems = {}
            self.timeitems = {}
            self._opensoundfile() # recreates the waveform and tickmarks

            # get the current text to break down
            normaltext = self.textentry.get()
            self.textentry.update_idletasks()


            # break it down
            self.phonetictext = spanish_breakdown.breakdowntext(normaltext)

            # add it to the canvas
            self.addwordstocanvas()
        else:
            tkMessageBox.showinfo(strings["No sound file"], strings["No sound file loaded - start a new Project first"])


    def simplifiedSpanishPhoneticBreakdown(self):
        """calls spanish_breakdown.simplifiedbreakdowntext to breakdown the sentence in self.textentry
        into a list of phonemes, then loads them onto the canvas

        If there is no soundfile already loaded, gives an error message instead
        """
        if hasattr(self, 'soundfile'):
            self.phonetictext = []
            self.display.delete(Tkinter.ALL) # clear the entire canvas
            self.phonetictext = []
            self.citems = {}
            self.timeitems = {}
            self._opensoundfile() # recreates the waveform and tickmarks

            # get the current text to break down
            normaltext = self.textentry.get()
            self.textentry.update_idletasks()


            # break it down
            self.phonetictext = spanish_breakdown.simplifiedbreakdowntext(normaltext)

            # add it to the canvas
            self.addwordstocanvas()
        else:
            tkMessageBox.showinfo(strings["No sound file"], strings["No sound file loaded - start a new Project first"])



    def phoneticBreakdown(self):
        """calls lookupword.breakdowntext to breakdown the sentence in self.textentry
        into a list of phonemes, then loads them onto the canvas

        If there is no soundfile already loaded, gives an error message instead
        """
        if hasattr(self, 'soundfile'):
            self.phonetictext = []
            self.display.delete(Tkinter.ALL) # clear the entire canvas
            self.phonetictext = []
            self.citems = {}
            self.timeitems = {}
            self._opensoundfile() # recreates the waveform and tickmarks

            # get the current text to break down
            normaltext = self.textentry.get()
            self.textentry.update_idletasks()

            # break it down
            self.phonetictext = lookupword.breakdowntext(normaltext, self)

            # add it to the canvas
            self.addwordstocanvas()
        else:
            tkMessageBox.showinfo(strings["No sound file"], strings["No sound file loaded - start a new Project first"])



# ==============================================================

    def newProject(self):
        """creates a new project (in memory) from scratch
        """
        fn = tkFileDialog.askopenfilename(filetypes=[('WAVE/RIFF sound files', '*.wav'),('AIFF, AIFF-C sound files', '*.aif'),('Sun AU sound files', '*.au')])
        if fn :
            self.soundfile = fn
            # clear any existing breakdown from canvas here
            self.display.delete(Tkinter.ALL)
            self.phonetictext = []
            self.citems = {}
            self.timeitems = {}
            self._opensoundfile()

            self.textentry.delete(0, Tkinter.END) # clear any existing stuff

    def openProject(self):
        """opens a saved project (see below), reads the preferences, 
        loads the sound file, puts the phonemes onto the canvas
        """
        fn = tkFileDialog.askopenfilename(filetypes=[('PAMELA project files', '*.pam')])
        if fn :
            self.projectname = fn
            self.projdata = pamfile.pamfile(self.projectname)
            self.fps = int(self.projdata['Preferences']['framespersecond'])
            self.framesperphoneme = int(self.projdata['Preferences']['framesperphoneme'])
            self.pixelspersecond = self.pixelsperframe * self.fps
            self.translation_file = self.projdata['Preferences']['translationfile']
            self.soundfile = self.projdata['Preferences']['soundfile']

            prefgui.saveprefs(self.fps, self.framesperphoneme, self.translation_file)
            self.readpreferences()

            if os.path.exists(self.soundfile):
                # clear any existing breakdown from canvas here
                self.display.delete(Tkinter.ALL)
                self.phonetictext = []
                self.citems = {}
                self.timeitems = {}
                self._opensoundfile()

                self.textentry.delete(0, Tkinter.END) # clear any existing stuff

                self.text = self.projdata['Preferences']['textdata']
                self.textentry.insert(Tkinter.END, self.text)

                phoneme = self.projdata['Speech']
                phonemetimes = phoneme.keys()
                phonemetimes.sort()
                for frame in phonemetimes :
                    if frame == '__name__' :
                        pass
                    else :
                        self.addphonemetocanvas(phoneme[frame], string.atoi(frame))
                self.parent.title("PAMELA - %s" % self.projectname)
            else:
                tkMessageBox.showinfo(strings["Error"], strings["Sound file path in project file is incorrect"])

    def saveProject(self):
        """saves a project that has been given a name, including preferences for this project, 
        the soundfile used, and the current phonemes on the canvas

        if there is no project name, calls saveProjectAs to get a project name, which calls this function again to save things
        """
        if not hasattr(self, 'projectname'):
            self.saveProjectAs()
        else :
            self.projdata = pamfile.pamfile(self.projectname)
            self.projdata['Preferences'] = {}
            self.projdata['Preferences']['framespersecond'] = self.fps
            self.projdata['Preferences']['framesperphoneme'] = self.framesperphoneme
            self.projdata['Preferences']['translationfile'] = self.translation_file
            self.projdata['Preferences']['soundfile'] = self.soundfile
            self.projdata['Preferences']['textdata'] = self.textentry.get()


            frametime, phoneme = 0,1
            self.projdata['Speech'] = {}
            phonemes = self.timeitems.values()
            phonemes.sort()
            for phonemeentry in phonemes :
                if phonemeentry[phoneme] != "end":
                    self.projdata['Speech'][int(phonemeentry[frametime])] = phonemeentry[phoneme]
            self.projdata.save(self.projectname)
            self.parent.title("PAMELA - %s" % self.projectname)


    def saveProjectAs(self):
        """gets a project name, then calls saveProject to do the actual project saving work
        """
        fn = tkFileDialog.asksaveasfilename(defaultextension='.pam', filetypes=[('PAMELA project files', '*.pam')])
        if fn :
            self.projectname = fn
            # print self.projectname
            self.saveProject()

# ==============================================================

    def pixelstoframes(self, eachframe):
        """converts (xpos pixel position, phoneme) pair into a [time, phoneme] minilist
        """
        return [int(eachframe[0] * self.fps / self.pixelspersecond)+1, eachframe[1]]

    def exportAction(self):
        """converts timeitems into a list of (time, phoneme) lists
        asks for an output name
        calls export.exportClass to export the list to a Moho Switch data file
        """
        speech = self.timeitems.values()
        speech = map(self.pixelstoframes, speech)
        speech.sort()
        fn = tkFileDialog.asksaveasfilename(defaultextension='.dat', filetypes=[('Moho Switch data files', '*.dat')], initialdir=".")
        if fn :
            e = export.exportClass(speech, self.translation_file, fn)

# ==============================================================


    def quit(self):
        try:
            if self.snd :  # get rid of the loaded sound if it exists
                self.snd.destroy()
        except :
            pass
        if tkMessageBox.askyesno(strings["Confirm Quit"], strings["Really quit ?"]):
            Tkinter.Frame.quit(self)
            self.destroy()
            # self.parent.destroy()


# ==============================================================

def nice_exit():
        v.quit()
        # del root

if __name__ == '__main__' : 
    root =  Tkinter.Tk()
    # root.iconbitmap('chip.ico')
    root.protocol("WM_DELETE_WINDOW", nice_exit)
    if mysound.useSnack :
        import tkSnack
        tkSnack.initializeSnack(root)
    v = PAMELA(root)
    root.mainloop()
    root.destroy()
    del root

