#!/usr/local/bin/python

# Modul TVariable
"""Defines TVariable, TStringVar, ... which extend Tkinter.Variable, 
Tkinter.StringVar,... by methods trace_read, trace_write to trace read
and write of variables.

Written by Nils Fischbeck (fischbec@tiger.informatik.hu-berlin.de)

snaffled from comp.lang.python
Subject: Re: Tkinter: callback when optionMenu changes?
Date: 1996/07/26
Message-ID: <pxzn30n1ke2.fsf@tiger.informatik.hu-berlin.de>#1/1

in reply to Tessa Lau

> I have several optionMenu widgets in my application, and I need to perform
> some actions when the value selected in on optionMenu changes.  A dejanews
> search of the tcl newsgroup suggested putting a write trace on the variable
> being set by the optionMenu; how can I do this in python?

Tcl allows to attach callbacks on variables whenever they are written
or read or unset. The tcl command is "trace variable varname [wru]". This is
currently not supported by Tkinter.Variable. I wrote a small module
which defines classes TVariable, TStringVar, TIntVar, ... which have a
method

trace_read(yourCallBack) and trace_write(yourCallBack).

The function yourCallBack is called with one Argument: the TVariable
which was written or read.


"""

from Tkinter import Variable, Misc

class TVariable(Variable, Misc):
    def __init__(self, master = None):
        Variable.__init__(self, master)
        self.tk = self._tk

    def trace_read(self,func=None):
        if func:
            print self.tk.call("info","vars")
            name = self._register(func, self._substitute)
            self._tk.call('trace','variable',self._name,'r',name)
            
    def trace_write(self,func=None):
        if func:
            name = self._register(func, self._substitute)
            self._tk.call('trace','variable',self._name,'w',name)
            
    def _substitute(self, *args):
        return self,
    

class TStringVar(TVariable):
        def __init__(self, master=None):
                TVariable.__init__(self, master)
        def get(self):
                return self._tk.globalgetvar(self._name)

class TIntVar(TVariable):
        def __init__(self, master=None):
                TVariable.__init__(self, master)
        def get(self):
                return self._tk.getint(self._tk.globalgetvar(self._name))

class TDoubleVar(TVariable):
        def __init__(self, master=None):
                TVariable.__init__(self, master)
        def get(self):
                return self._tk.getdouble(self._tk.globalgetvar(self._name))

class TBooleanVar(TVariable):
        def __init__(self, master=None):
                TVariable.__init__(self, master)
        def get(self):
                return self._tk.getboolean(self._tk.globalgetvar(self._name))
