import struct, winsound, string, os.path
# import sndhdr, audioop

# currently only valid for mono 8-bit PCM-format files (but should work with WAV, AU, AIF)
# currently plays sounds and sound fragments using winsound module, MS-Windows only

useSnack = 1
if useSnack:
    import tkSnack

from Tkinter import *

class mySound:

    def __init__(self, filename) :
        self.readsound(filename)

    def readsound(self, filename) :
        if useSnack:
            self.sounddata = tkSnack.Sound()
            self.sounddata.read(filename)
            self.framerate = self.sounddata["frequency"] # self.sounddata.cget("frequency")
        else:
            extension = string.lower(os.path.splitext(filename)[1])
            if extension == '.wav' :
                import wave
                f = wave.open(filename, 'r')
            elif (extension == '.aiff' or extension == '.aif') :
                import aifc
                f = aifc.open(filename, 'r')
            elif (extension == '.au') :
                import sunau
                f = sunau.open(filename, 'r')
            else :
                return None
            self.nchannels = f.getnchannels() # 1 = mono, 2 = stereo
            self.sampwidth =  f.getsampwidth() # bytes in each sample
            self.framerate =  f.getframerate() # samples per second
            self.nframes = f.getnframes() # number of sample frames
            self.length = float(f.getnframes())/f.getframerate() # length in seconds
            self.comptype = f.getcomptype() # compression, 1= PCM
            self.compname = f.getcompname() # compression name
            self.params = f.getparams() # tuple of all the above

            self.sounddata = f.readframes(f.getnframes())
            f.close()

    def destroy(self) :
        if useSnack:
            self.sounddata.destroy()
        else :
            del self.sounddata

    def drawsound(self, display, pixelspersecond) :
        if useSnack:
            c = tkSnack.createWaveform(display, 0,0, sound=self.sounddata, height=128, zerolevel = 1, pixelspersecond=pixelspersecond)
            self.width = display.bbox(c)[2]
        else:
            # 128 and 60 probably only work for 8-bit files
            # handling of sounddata probably only works for mono
            i = 0
            framesperpixel = int(self.framerate / pixelspersecond)
            for n in xrange(0, self.nframes, framesperpixel) :
                i = i+1
                ml = self.sounddata[n:n+framesperpixel]
                maxml = ord(max(ml))
                y = maxml-128
                display.create_line(i, 60-y, i, 60+y)
            self.width = self.nframes / framesperpixel

    def playsound(self, start=0, end=0, blocking=None) :
        if useSnack:
            if end == 0:
                self.sounddata.play()
            else:
                if blocking:
                    self.sounddata.play(start=start, end=end, blocking=1)
                else:
                    self.sounddata.play(start=start, end=end)
        else:
            "creates an in-memory sound-file and plays it"
            # current setup probably only works for 8-bit mono
            if end == 0:
                end = len(self.sounddata)
            datasize = end-start
            filesize=datasize+50
            header = []

            header.append('RIFF')
            filesize = struct.pack('<L', filesize)
            header.append(filesize)
            header.append('WAVEfmt ')
            FormatTag = self.comptype # 1 = PCM
            Channels = self.nchannels # 1 = mono
            SamplesPerSec = self.framerate # 11025, 22050, 44100
            if self.comptype == 'NONE' :
                BitsPerSample = self.sampwidth*8
                FormatTag = 1
            else :
                unknown_wav_format = "Unknown WAV format"
                raise unknown_wav_format, self.comptype, self.compname
            BlockAlign = Channels * (BitsPerSample/8)
            AvgBytesPerSec = BlockAlign * SamplesPerSec
            fmtchunksize = struct.calcsize('<LHHLLH')
            fmtchunk = struct.pack('<LHHLLH', fmtchunksize, 
                FormatTag, Channels, SamplesPerSec, AvgBytesPerSec, BlockAlign)
            header.append(fmtchunk)
            header.append(struct.pack('<L', BitsPerSample))
            header.append('fact')
            header.append(struct.pack('<L', 4))
            header.append(struct.pack('<L', datasize))
            header.append('data')
            header.append(struct.pack('<L', datasize))

            # probably only set up for 8-bit mono at the moment
            header.append(self.sounddata[start:end])
            buffer = string.join(header, '')
            f = open('C:/data/moho/lipsync/mytest.wav', 'wb',)
            f.write(buffer)
            f.close()
            winsound.PlaySound(buffer, winsound.SND_MEMORY)


class testsound :

    def __init__(self, filename, parent=None):    # an extended frame
        self.snd = mySound(filename)
        self.mainframe = Frame(parent)
        self.mainframe.pack()
        self.sb = Scrollbar(self.mainframe, orient="horizontal", jump=1)
        self.display = Canvas(self.mainframe, height=120, bg="white", xscrollcommand=self.sb.set)
        self.sb.config(command=self.display.xview)
        self.snd.drawsound(self.display, 320)
        self.sb.pack(side=BOTTOM, expand=YES, fill=X)
        self.display.pack(expand=YES, fill=BOTH)
        self.display.config(scrollregion=self.display.bbox(ALL))
        # self.snd.playsound(10000, 20000)
        self.snd.playsound(0, 0)

def nice_exit():
        t.quit()

if __name__ == '__main__' : 
    root =  Tk()
    root.protocol("WM_DELETE_WINDOW", nice_exit)
    if useSnack :
        tkSnack.initializeSnack(root)
    t = testsound('crumbs.wav', root)
    root.mainloop()
    # root.destroy()
    del root
